#! /usr/bin/env python

#
# call from package root for example 
# 
#    cd kdelibs 
#    python bksys/configure.py    
#

import os,sys,types

from os.path import join
import sys
sys.path = ['scons-local-0.96.91'] + sys.path; 
import SCons

import SCons.Tool
import SCons.Util
from SCons.Script.SConscript import SConsEnvironment
from SCons.Options import Options, PathOption
from SCons import Environment

env = Environment.Environment()
# TODO: put on a central place 

class PackageConfiguration:
	def __init__(self,env):
		self.build_dir = 'build/'
		self.cache_dir = 'cache/'
		self.headers = {}
		self.functions = {}
		self.types = {}
		self.libraries = {}
		self.options = {}
		self.uselibs = {}
		self.modules = {}
		self.headerIsOpen = 0
		self.env = env
		self.config_h = ['config_header.h']
		self.cache = ''
		cachefile=self.cache_dir+'configure.cache.py'
		self.readCache(cachefile)
		self.openCache(cachefile)
				
		# hack to make scons happy 
		SCons.Script.sconscript_reading = 1
		self.sconsConf = env.Configure()

	def readCache(self,filename):
		headers = {}
		functions = {}
		types = {}
		libraries = {}
		if os.path.exists(filename):
			execfile(filename)
			self.headers = headers
			self.functions = functions
			self.types = types
			self.libraries = libraries
			return 1
		else:
			return 0
	
	def openCache(self,filename):
		self.cache =open(filename, 'a')

	def writeCache(self,type,name,ret):
		if not self.cache:
			return 0
			
		if ret:
			val = "1"
		else:
			val = "0"
		print type + " " + name + " " + val
		if type == 'header':
			self.cache.write('headers[\'' + name + '\'] = ' + val + '\n')
		elif type == 'function':
			self.cache.write('functions[\'' + name + '\'] = ' + val + '\n')
		elif type == 'type':
			self.cache.write('types[\'' + name + '\'] = ' + val + '\n')
		elif type == 'library':
			self.cache.write('libraries[\'' + name + '\'] = ' + val + '\n')
		return 1

	def closeCache(self):
		if self.cache:
			self.cache.close()

	##  
	# \private
	def openConfigHeader(self,filename):
		if self.headerIsOpen: 
			self.closeConfigHeader()

		# TODO: add definitions if file exist, save file pointers 
		self.headerIsOpen = 1
		self.dest=open(filename, 'w')
		s = '_' + filename.upper().replace('/','_').replace('.','_').replace('-','_')
		self.dest.write('/* configuration created by bksys */\n\n')
		self.dest.write('#ifndef %s\n#define %s 1\n\n' % (s,s))
		self.dest.write('#include <kdelibs_export.h>\n\n')
		self.headerLastLine = '#endif /* %s */\n' % (s)

	##  
	# \private
	def closeConfigHeader(self):
		self.dest.write(self.headerLastLine)
		self.dest.close()
		self.headerIsOpen = 0

	##  
	# \private
	def writeConfigHeader(self,content):
		if self.headerIsOpen: 
			self.dest.write(content)

	def buildConfigHeaderEntry(self,name,ret):
		s = 'HAVE_' + name.upper().replace('/','_').replace('.','_')				
		if ret:
			content = "#ifndef %s\n#define %s 1\n#endif\n\n" % (s,s)
		else:
			content = "/* #undef %s */\n\n" % (s)
		return content

	## emulates scons checking output for cached entries
	#
	def DisplayResult(self,type,name,language,ret):
		s = ["no","yes"]
		if language ==  None:
			lang = 'C'
		else: 
			lang = language
		if type == 'header':
			print "Checking for %s header file %s (cached) ... %s" % (lang, name,s[ret])
		elif type == 'function':
			print "Checking for function %s() (cached) ... %s" % (name,s[ret])
		elif type == 'type':
			print "Checking for type %s (cached) ... %s" % (name,s[ret])
		elif type == 'library':
			print "Checking for library %s (cached) ... %s" % (name,s[ret])


	def setConfigHeader(self,filename):
		self.config_h += [filename]
		print "setConfigHeader " + filename
		self.openConfigHeader(self.build_dir+filename)
		
	def CheckHeader(self,header,include_quotes='<>',language = None):
		ret = 0
		if not self.headers.has_key(header):
			ret = self.sconsConf.CheckHeader(header,include_quotes,language)
			self.headers[header] = ret
			self.headers[header+'_def'] = self.buildConfigHeaderEntry(header,ret)
			self.writeCache('header',header,ret)
		else:
			ret = self.headers[header]
			if not self.headers.has_key(header+'_def'):
				self.headers[header+'_def'] = self.buildConfigHeaderEntry(header,ret)
			self.DisplayResult('header',header,language,ret)
		self.writeConfigHeader(self.headers[header+'_def'])
		return ret
		
	def CheckFunc(self,function_name, header = None, language = None):
		ret = 0
		if not self.functions.has_key(function_name):
			ret = self.sconsConf.CheckFunc(function_name,header,language)
			self.functions[function_name] = ret
			self.functions[function_name+'_def'] = self.buildConfigHeaderEntry(function_name,ret)
			self.writeCache('function',function_name,ret)
		else:
			ret = self.functions[function_name]
			if not self.functions.has_key(function_name+'_def'):
				self.functions[function_name+'_def'] = self.buildConfigHeaderEntry(function_name,ret)
			self.DisplayResult('function',function_name,language,ret)

		self.writeConfigHeader(self.functions[function_name+'_def'])
		return ret

	def CheckType(self,type_name, includes = "", language = None):
		ret = 0
		if not self.types.has_key(type_name):
			ret = self.sconsConf.CheckType(type_name,includes,language)
			self.types[type_name] = ret
			self.types[type_name+'_def'] = self.buildConfigHeaderEntry(type_name,ret)
			self.writeCache('type',type_name,ret)
		else:
			ret = self.types[type_name]
			if not self.types.has_key(type_name+'_def'):
				self.types[type_name+'_def'] = self.buildConfigHeaderEntry(type_name,ret)
			self.DisplayResult('type',type_name,language,ret)

		self.writeConfigHeader(self.types[type_name+'_def'])
		return ret
		
	def CheckLib(self,library = None, symbol = "main", autoadd = 1,
             					header = None, language = None):
		ret = 0
		if not self.libraries.has_key(library): 
			ret = self.sconsConf.CheckLib(library, symbol, header = header,
                                        language = language, autoadd = autoadd) 
			self.libraries[library] = ret 
			self.libraries[library+'_def'] = self.buildConfigHeaderEntry(library,ret) 
			self.writeCache('library',library,ret)
		else: 
			ret = self.libraries[library]
			if not self.libraries.has_key(library+'_def'): 
				self.libraries[library+'_def'] = self.buildConfigHeaderEntry(library,ret) 
			self.DisplayResult('library',library,language,ret)

		self.writeConfigHeader(self.libraries[library+'_def'])
		return ret
		
	def CheckLibWithHeader(self, library, header, language,
                       call = "main();", autoadd = 1):		
		ret = 0
		if not self.libraries.has_key(library): 
			ret = self.sconsConf.CheckLibWithHeader(library, header, language, 
																						call = call, autoadd = autoadd)
			self.libraries[library] = ret 
			self.libraries[library+'_def'] = self.buildConfigHeaderEntry(library,ret) 
			self.writeCache('library',library,ret)
		else: 
			ret = self.libraries[library]
			if not self.libraries.has_key(library+'_def'): 
				self.libraries[library+'_def'] = self.buildConfigHeaderEntry(library,ret) 
			self.DisplayResult('library',library,language,ret)

		self.writeConfigHeader(self.libraries[library+'_def'])
		return ret

	def Finish(self):
		self.closeCache()
		return self.sconsConf.Finish()

	

def SConfigure(dict,env):
	build_dir  = '.'
	cache_dir  = 'cache'+os.sep
	
	# process the options
	for key in dict.keys():
		if key == 'builddir':  build_dir  = dict[key]
		elif key == 'cache':     cache_dir  = dict[key] + os.sep
	
	conf = PackageConfiguration(env)
	verbose = 1
		
	# walk through available directory an see if there are any SConfigure files 
	if verbose: 
		print 'SConfigure(s) found in '
	for root, dirs, files in os.walk('.'):
	
		# skip svn or cvs directories 
		if '.svn' in root or 'CVS' in root:
			continue
	
		# got one
		if 'SConfigure' in files:
			if verbose:
				print root
			_file_ = open(root + os.sep + 'SConfigure', "r")
			# conf is given as global object 
			exec _file_
			conf.closeConfigHeader()
	
	conf.Finish()
	
	# TODO: how to proceed with env setting like kdecore/SConfigure	

	sconf_global = None
	return conf.config_h



## CONFIGURATION: write the config.h files
def write_config_h(lenv):

	print lenv['_CONFIG_H_']
	# put the files into the builddir
	dest=open(lenv.join(lenv['_BUILDDIR_'], 'config.h'), 'w')
	dest.write('#ifndef BKSYS_CONFIG_H\n#define BKSYS_CONFIG_H\n\n/* defines are added below */\n')

	# write the config.h including all others
	if lenv.has_key('_CONFIG_H_'):
		for file in lenv['_CONFIG_H_']: 
			dest.write('#include "config-%s.h"\n' % file)

	if env['CONFIG_HEADER']:
		for file in env['CONFIG_HEADER']: 
			dest.write('#include "%s"\n' % file)
	dest.write('\n\n#endif\n')
	dest.close()

	lenv.pprint('GREEN','configuration done - run scons to compile now')
	lenv.Exit(0)


def configure(dict):
	cp_method  = 'soft-copy'
	tool_path  = ['bksys']
	build_dir  = '.'
	cache_dir  = 'cache'+os.sep
	config_h   = 0
	bootstrap  = 0
	want_rpath = 1
	use_colors = 0
	
	# process the options
	for key in dict.keys():
		if   key == 'modules':   mytools    = dict[key].split()
		elif key == 'builddir':  build_dir  = dict[key]
		elif key == 'config.h':  config_h   = 1
		elif key == 'cp_method': cp_method  = dict[key]
		elif key == 'rpath':     want_rpath = dict[key]
		elif key == 'bootstrap': bootstrap  = 1
		elif key == 'cachedir':  cache_dir  = dict[key]
		elif key == 'colorful':  use_colors = dict[key]
		elif key == 'cache':     cache_dir  = dict[key] + os.sep
		else: print 'unknown key: '+key
	
	# rpath flags are wrong (and pointless) on darwin
	if sys.platform == 'darwin':
		want_rpath = 0
	
	# make sure the build dir is available
	# TODO what if it is in a non-existing subdirectory ? (ita)
	if not os.path.exists(build_dir): os.mkdir(build_dir)
	
	# now build the environment
	if bootstrap:
		env = Environment.Environment( ENV=os.environ, BOOTSTRAP_KDE=1,
			_BUILDDIR_=build_dir, CACHEDIR=cache_dir, _USECOLORS_=use_colors,
			tools=mytools, toolpath=tool_path )
	else:
		env = Environment.Environment( ENV=os.environ, _BUILDDIR_=build_dir,
			_USECOLORS_=use_colors, CACHEDIR=cache_dir,
			tools=mytools, toolpath=tool_path )
	

	if 0:
		sconf_global = None
		conf = PackageConfiguration()
		verbose = 1
		
		# walk through available directory an see if there are any SConfigure files 
		if verbose: 
			print 'SConfigure(s) found in '
		for root, dirs, files in os.walk('.'):
		
			# skip svn or cvs directories 
			if '.svn' in root or 'CVS' in root:
				continue
		
			# got one
			if 'SConfigure' in files:
				if verbose:
					print root
				_file_ = open(root + os.sep + 'SConfigure', "r")
				# conf is given as global object 
				exec _file_
				conf.closeConfigHeader()
		
		
		config_headers = conf.config_h
		
		conf.Finish()
		
	# now the post-configuration
	# write the config.h if we were asked to, then quit
	write_config_h(env)

config = {
	'modules'  : 'generic lowlevel compiler libxml kde4 libz libpng libjpeg libagg libart2 libidn libpcre openssl fam libgif libbz2 yacc',
	'cache'    : 'cache', # where to put all cache files 
	'builddir' : 'build', # put all object files under 'build/'
	'config.h' : 1, # mechanism should be ok
	'rpath'    : 1, # incomplete
	'bootstrap': 1, # incomplete
	'colorful' : not os.environ.has_key('NOCOLORS'), # only with scons >= 0.96.91 - now miniscons
}


env['CONFIG_HEADER']=SConfigure(config,env)

env=configure(config)

